from flask import Blueprint, request
from flask_restful import Api, Resource, abort
from marshmallow import ValidationError

from portal.rest import UserSchema, SubmissionSchema, RoleSchema, CourseSchema, GroupSchema, ReviewSchema
from portal.service import service
from portal.database.models import User
from portal.tools.logging import log
from portal.tools.decorators import error_handler

users = Blueprint('users', __name__, url_prefix='/users')
users_api = Api(users)

user_schema = UserSchema()
users_schema = UserSchema(many=True, only=('id', 'username', 'uco', 'email'))
submissions_schema = SubmissionSchema(many=True, only=('id', 'state', 'project', 'scheduled_for'))
roles_schema = RoleSchema(many=True, only=('id', 'name', 'description', 'course'))
courses_schema = CourseSchema(many=True, only=('id', 'name', 'codename'))
groups_schema = GroupSchema(many=True, only=('id', 'name', 'course'))
reviews_schema = ReviewSchema(many=True)


class UserResource(Resource):
    @error_handler
    def get(self, uid):
        log.info(f"GET to {request.url}")
        user = service.find_user(uid)
        return user_schema.dump(user)[0], 200

    @error_handler
    def put(self, uid):
        log.info(f"PUT to {request.url}")
        user = service.find_user(uid)

        json_data = request.get_json()
        if not json_data:
            abort(400, message='No data provided for user update.')
        try:
            data = user_schema.load(json_data)[0]
        except ValidationError as err:
            log.info(f"Validation failed on: {err.messages}")
            return abort(422, errors=err.messages)  # TODO: flash, process errors

        user.name = data['name']
        user.email = data['email']
        user.uco = data['uco']
        user.username = data['username']
        user.is_admin = data['is_admin']
        # TODO all; care for password
        service.write_entity(user)
        log.debug(f"Updated user {uid} to {user}.")
        return '', 204

    @error_handler
    def delete(self, uid):
        log.info(f"DELETE to {request.url}")
        user = service.find_user(uid)
        service.delete_entity(user)
        return '', 204


class UserList(Resource):
    @error_handler
    def get(self):
        log.info(f"GET to {request.url}")
        users = User.query.all()
        return users_schema.dump(users)[0], 200

    @error_handler
    def post(self):
        log.info(f"POST to {request.url}")
        json_data = request.get_json()
        if not json_data:
            abort(400, message='No data provided for user creation.')
        try:
            data = user_schema.load(json_data)[0]
        except ValidationError as err:
            log.info(f"Validation failed on: {err.messages}")
            return abort(422, errors=err.messages)  # TODO: flash, process errors

        admin = data.get('is_admin') or False
        new_user = User(uco=data['uco'], email=data['email'], username=data['username'], is_admin=admin)
        service.write_entity(new_user)
        log.debug(f"Created user={User}")
        return user_schema.dump(new_user)[0], 201


class UserSubmissionList(Resource):
    @error_handler
    def get(self, uid):
        log.info(f"GET to {request.url}")
        user = service.find_user(uid)
        return submissions_schema.dump(user.submissions)[0], 200


class UserRoleList(Resource):
    @error_handler
    def get(self, uid):
        log.info(f"GET to {request.url}")
        user = service.find_user(uid)
        return roles_schema.dump(user.roles)[0], 200


class UserCourseList(Resource):
    @error_handler
    def get(self, uid):
        log.info(f"GET to {request.url}")
        user = service.find_user(uid)
        return courses_schema.dump(user.courses)[0], 200


class UserGroupList(Resource):
    @error_handler
    def get(self, uid):
        log.info(f"GET to {request.url}")
        user = service.find_user(uid)
        return groups_schema.dump(user.groups)[0], 200


class UserReviewList(Resource):
    @error_handler
    def get(self, uid):
        log.info(f"GET to {request.url}")
        user = service.find_user(uid)
        reviews = []
        for item in user.review_items:
            if item.review not in reviews:
                reviews.append(item.review)
        return reviews_schema.dump(reviews)[0], 200


users_api.add_resource(UserResource, '/<string:uid>')
users_api.add_resource(UserList, '/')
users_api.add_resource(UserSubmissionList, '/<string:uid>/submissions')
users_api.add_resource(UserRoleList, '/<string:uid>/roles')
users_api.add_resource(UserCourseList, '/<string:uid>/courses')
users_api.add_resource(UserGroupList, '/<string:uid>/groups')
users_api.add_resource(UserReviewList, '/<string:uid>/reviews')

